@@ -1,23 +1,29 @@
 from datetime import datetime
 import pytest
+from respx import MockRouter
 
 import httpx
 
-from .thermostat import Device, SetpointMode, SystemMode, thermostat_client
 from .errors import RequestClientException, UnsuportedArgumentsException
+from .thermostat import Device, SetpointMode, SystemMode, thermostat_client
+from .token import Token
 
-token = {
+token = Token({
     "access_token": "12345",
     "refresh_token": "abcde",
     "expires_at": "",
-    "expires_in": ""
-}
+})
 
 get_thermostats_data_request = {
     "device_type": "NAVaillant",
     "access_token": "12345",
 }
 
+get_thermostats_data_refreshed_request = {
+    "device_type": "NAVaillant",
+    "access_token": "67890",
+}
+
 get_thermostats_data_response = {
     "status": "ok",
     "body": {
@@ -69,16 +75,44 @@ set_minor_mode_response = {
     "status": "ok",
 }
 
+refresh_token_request = {
+    "grant_type": "refresh_token",
+    "client_id": "client",
+    "client_secret": "secret",
+    "refresh_token": "abcde",
+}
+
+refresh_token_response = {
+    "access_token": "67890",
+    "refresh_token": "fghij",
+    "expires_at": "",
+}
+
 @pytest.mark.asyncio
 class TestThermostat:
-    async def test_async_get_thermostats_data__invalid_request_params__raises_error(self, respx_mock):
+    async def test_async_get_thermostats_data__invalid_request_params__raises_error(self, respx_mock: MockRouter):
         respx_mock.post("https://api.netatmo.com/api/getthermostatsdata", data=get_thermostats_data_request).respond(400)
 
         async with thermostat_client("", "", token, None) as client:
             with pytest.raises(RequestClientException):
                 await client.async_get_thermostats_data()
 
-    async def test_async_get_thermostats_data__server_errors__retry_until_success(self, respx_mock):
+    async def test_async_get_thermostats_data__server_errors__retry_until_success(self, respx_mock: MockRouter):
+        respx_mock.post("https://api.netatmo.com/api/getthermostatsdata", data=get_thermostats_data_request).respond(401)
+        respx_mock.post("https://api.netatmo.com/oauth2/token", data=refresh_token_request).respond(200, json=refresh_token_response)
+        respx_mock.post("https://api.netatmo.com/api/getthermostatsdata", data=get_thermostats_data_refreshed_request).respond(200, json=get_thermostats_data_response)
+
+        async with thermostat_client(refresh_token_request["client_id"], refresh_token_request["client_secret"], token, None) as client:
+            devices = await client.async_get_thermostats_data()
+
+            expected_devices = get_thermostats_data_response["body"]["devices"]
+
+            assert respx_mock.calls.call_count == 3
+            assert len(devices) == len(expected_devices)
+            for x in zip(devices, expected_devices):
+                assert x[0] == Device(**x[1])
+
+    async def test_async_get_thermostats_data__unauthorized_errors__succeed_after_refreshing_token(self, respx_mock: MockRouter):
         respx_mock.post("https://api.netatmo.com/api/getthermostatsdata", data=get_thermostats_data_request).mock(side_effect=[
             httpx.Response(500),
             httpx.Response(200, json=get_thermostats_data_response),
@@ -94,7 +128,7 @@ class TestThermostat:
             for x in zip(devices, expected_devices):
                 assert x[0] == Device(**x[1])
 
-    async def test_async_get_thermostats_data__valid_request_params__returns_valid_device_list(self, respx_mock):
+    async def test_async_get_thermostats_data__valid_request_params__returns_valid_device_list(self, respx_mock: MockRouter):
         respx_mock.post("https://api.netatmo.com/api/getthermostatsdata", data=get_thermostats_data_request).respond(200, json=get_thermostats_data_response)
 
         async with thermostat_client("", "", token, None) as client:
@@ -106,14 +140,14 @@ class TestThermostat:
             for x in zip(devices, expected_devices):
                 assert x[0] == Device(**x[1])
 
-    async def test_async_set_system_mode__invalid_request_params__raises_error(self, respx_mock):
+    async def test_async_set_system_mode__invalid_request_params__raises_error(self, respx_mock: MockRouter):
         respx_mock.post("https://api.netatmo.com/api/setsystemmode", data=set_system_mode_request).respond(400)
 
         async with thermostat_client("", "", token, None) as client:
             with pytest.raises(RequestClientException):
                 await client.async_set_system_mode(set_system_mode_request["device_id"], set_system_mode_request["module_id"], SystemMode.SUMMER)
 
-    async def test_async_set_system_mode__server_errors__retry_until_success(self, respx_mock):
+    async def test_async_set_system_mode__server_errors__retry_until_success(self, respx_mock: MockRouter):
         respx_mock.post("https://api.netatmo.com/api/setsystemmode", data=set_system_mode_request).mock(side_effect=[
             httpx.Response(500),
             httpx.Response(200, json=set_system_mode_response),
@@ -124,20 +158,20 @@ class TestThermostat:
 
             assert respx_mock.calls.call_count == 2
 
-    async def test_async_set_system_mode__valid_request_params__doesnt_raise_error(self, respx_mock):
+    async def test_async_set_system_mode__valid_request_params__doesnt_raise_error(self, respx_mock: MockRouter):
         respx_mock.post("https://api.netatmo.com/api/setsystemmode", data=set_system_mode_request).respond(200, json=set_system_mode_response)
 
         async with thermostat_client("", "", token, None) as client:
             await client.async_set_system_mode(set_system_mode_request["device_id"], set_system_mode_request["module_id"], SystemMode.SUMMER)
 
-    async def test_async_set_minor_mode__invalid_request_params__raises_error(self, respx_mock):
+    async def test_async_set_minor_mode__invalid_request_params__raises_error(self, respx_mock: MockRouter):
         respx_mock.post("https://api.netatmo.com/api/setminormode", data=set_minor_mode_request).respond(400)
 
         async with thermostat_client("", "", token, None) as client:
             with pytest.raises(RequestClientException):
                 await client.async_set_minor_mode(set_minor_mode_request["device_id"], set_minor_mode_request["module_id"], SetpointMode.AWAY, True)
 
-    async def test_async_set_minor_mode__server_errors__retry_until_success(self, respx_mock):
+    async def test_async_set_minor_mode__server_errors__retry_until_success(self, respx_mock: MockRouter):
         respx_mock.post("https://api.netatmo.com/api/setminormode", data=set_minor_mode_request).mock(side_effect=[
             httpx.Response(500),
             httpx.Response(200, json=set_minor_mode_response),
@@ -185,7 +219,7 @@ class TestThermostat:
                     setpoint_temp=25,
                 )
 
-    async def test_async_set_minor_mode__activate_manual_with_temp_and_endtime__executes_successfully(self, respx_mock):
+    async def test_async_set_minor_mode__activate_manual_with_temp_and_endtime__executes_successfully(self, respx_mock: MockRouter):
         endtime = round(datetime.now().timestamp()) + 1 # unix epoch timestamp
 
         set_minor_mode_request = {
@@ -248,7 +282,7 @@ class TestThermostat:
                     setpoint_endtime=datetime.now(),
                 )
 
-    async def test_async_set_minor_mode__deactivate_manual_without_temp_and_endtime__executes_successfully(self, respx_mock):
+    async def test_async_set_minor_mode__deactivate_manual_without_temp_and_endtime__executes_successfully(self, respx_mock: MockRouter):
         set_minor_mode_request = {
             "device_id": "device",
             "module_id": "module",
@@ -271,7 +305,7 @@ class TestThermostat:
                 False,
             )
 
-    async def test_async_set_minor_mode__activate_away_without_temp_and_endtime__executes_successfully(self, respx_mock):
+    async def test_async_set_minor_mode__activate_away_without_temp_and_endtime__executes_successfully(self, respx_mock: MockRouter):
         set_minor_mode_request = {
             "device_id": "device",
             "module_id": "module",
@@ -294,7 +328,7 @@ class TestThermostat:
                 True,
             )
 
-    async def test_async_set_minor_mode__activate_away_without_temp__executes_successfully(self, respx_mock):
+    async def test_async_set_minor_mode__activate_away_without_temp__executes_successfully(self, respx_mock: MockRouter):
         endtime = round(datetime.now().timestamp()) + 1 # unix epoch timestamp
         
         set_minor_mode_request = {
@@ -378,7 +412,7 @@ class TestThermostat:
                     setpoint_endtime=datetime.now(),
                 )
 
-    async def test_async_set_minor_mode__deactivate_away_without_temp_and_endtime__executes_successfully(self, respx_mock):
+    async def test_async_set_minor_mode__deactivate_away_without_temp_and_endtime__executes_successfully(self, respx_mock: MockRouter):
         set_minor_mode_request = {
             "device_id": "device",
             "module_id": "module",
@@ -411,7 +445,7 @@ class TestThermostat:
                     True,
                 )
 
-    async def test_async_set_minor_mode__activate_hwb_without_temp__executes_successfully(self, respx_mock):
+    async def test_async_set_minor_mode__activate_hwb_without_temp__executes_successfully(self, respx_mock: MockRouter):
         endtime = round(datetime.now().timestamp()) + 1 # unix epoch timestamp
 
         set_minor_mode_request = {
@@ -495,7 +529,7 @@ class TestThermostat:
                     setpoint_endtime=datetime.now(),
                 )
 
-    async def test_async_set_minor_mode__deactivate_hwb_without_temp_and_endtime__executes_successfully(self, respx_mock):
+    async def test_async_set_minor_mode__deactivate_hwb_without_temp_and_endtime__executes_successfully(self, respx_mock: MockRouter):
         set_minor_mode_request = {
             "device_id": "device",
             "module_id": "module",
